@@ -259,10 +259,20 @@ fake_device_error_test (void)
 	g_object_unref (device);
 }
 
+static void
+fill_pattern_cb (ArvBuffer *buffer, void *fill_pattern_data, guint32 exposure_time_us, guint32 gain, ArvPixelFormat pixel_format)
+{
+	gint *counter = fill_pattern_data;
+
+	(*counter)++;
+}
+
 static void
 fake_stream_test (void)
 {
 	ArvCamera *camera;
+	ArvDevice *device;
+	ArvFakeCamera *fake_camera;
 	ArvStream *stream;
 	ArvBuffer *buffer;
 	guint64 n_completed_buffers;
@@ -271,13 +281,22 @@ fake_stream_test (void)
 	gint n_input_buffers;
 	gint n_output_buffers;
 	gint payload;
+	gint counter = 0;
 
 	camera = arv_camera_new ("Fake_1");
 	g_assert (ARV_IS_CAMERA (camera));
 
+	device = arv_camera_get_device (camera);
+	g_assert (ARV_IS_DEVICE (device));
+
+	fake_camera = arv_fake_device_get_fake_camera (ARV_FAKE_DEVICE (device));
+	g_assert (ARV_IS_FAKE_CAMERA (fake_camera));
+
 	stream = arv_camera_create_stream (camera, NULL, NULL);
 	g_assert (ARV_IS_STREAM (stream));
 
+	arv_fake_camera_set_fill_pattern (fake_camera, fill_pattern_cb, &counter);
+
 	payload = arv_camera_get_payload (camera);
 	arv_stream_push_buffer (stream,  arv_buffer_new (payload, NULL));
 	arv_camera_set_acquisition_mode (camera, ARV_ACQUISITION_MODE_SINGLE_FRAME);
@@ -285,6 +304,10 @@ fake_stream_test (void)
 	buffer = arv_stream_pop_buffer (stream);
 	arv_camera_stop_acquisition (camera);
 
+	arv_fake_camera_set_fill_pattern (fake_camera, NULL, NULL);
+
+	g_assert_cmpint (counter, ==, 1);
+
 	g_assert (ARV_IS_BUFFER (buffer));
 
 	arv_stream_get_statistics (stream, &n_completed_buffers, &n_failures, &n_underruns);
@@ -312,6 +335,8 @@ main (int argc, char *argv[])
 
 	arv_enable_interface ("Fake");
 
+	arv_update_device_list ();
+
 	arv_set_fake_camera_genicam_filename (GENICAM_FILENAME);
 
 	g_test_add_func ("/fake/load-fake-camera-genicam", load_fake_camera_genicam_test);
